/*
  Simple example for receiving
  
  http://code.google.com/p/rc-switch/
  
  Need help? http://forum.ardumote.com
*/
#include <stdarg.h>
#include <RCSwitch.h>

#define R 3
#define G 5
#define B 6

RCSwitch mySwitch = RCSwitch();

const int buttonPin = 8;     // the number of the pushbutton pin
int buttonState;             // the current reading from the input pin
int lastButtonState = LOW;   // the previous reading from the input pin

// the following variables are long's because the time, measured in miliseconds,
// will quickly become a bigger number than can be stored in an int.
long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50;    // the debounce time; increase if the output flicke


const int devcode=0x11;

unsigned long lastval=0xffffff00;
boolean off=true;

void setup()
{
  Serial.begin(9600);
  Serial.println("Setup");
  mySwitch.enableReceive(0);  // Receiver on inerrupt 0 => that is pin #2
  pinMode(R, OUTPUT); 
  pinMode(G, OUTPUT);
  pinMode(B, OUTPUT);
  set(0,0,0,0);
  pinMode(buttonPin, INPUT);
  //http://arduino.cc/en/Tutorial/SecretsOfArduinoPWM
  //-> phase-correct PWM
  //TCCR2A = _BV(COM2A1) | _BV(COM2B1) | _BV(WGM20);
  //TCCR2B = _BV(CS22);
  //OCR2A = 180;
  //OCR2B = 50;
}

void button()
{
  // read the state of the switch into a local variable:
  int reading = digitalRead(buttonPin);

  // check to see if you just pressed the button
  // (i.e. the input went from LOW to HIGH),  and you've waited
  // long enough since the last press to ignore any noise:  

  // If the switch changed, due to noise or pressing:
  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }
 
  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:

    // if the button state has changed:
    if (reading != buttonState) {
      buttonState = reading;

      // only toggle the LED if the new button state is HIGH
      if (buttonState == HIGH) {
        if(off)
        {
          Serial.println("SWITCH ON");
          set(lastval);
        }
        else
        {
          Serial.println("SWITCH OFF");
          set(0,0,0,0);
        }
      }
    }
  }
 
  // save the reading.  Next time through the loop,
  // it'll be the lastButtonState:
  lastButtonState = reading;
}

void p(char *fmt, ... )
{
        char buf[128]; // resulting string limited to 128 chars
        va_list args;
        va_start (args, fmt );
        vsnprintf(buf, 128, fmt, args);
        va_end (args);
        Serial.print(buf);
}

void p(const __FlashStringHelper *fmt, ... )
{
  char buf[128]; // resulting string limited to 128 chars
  va_list args;
  va_start (args, fmt);
#ifdef __AVR__
  vsnprintf_P(buf, sizeof(buf), (const char *)fmt, args); // progmem for AVR
#else
  vsnprintf(buf, sizeof(buf), (const char *)fmt, args); // for the rest of the world
#endif
  va_end(args);
  Serial.print(buf);
}


void loop()
{
  button();
  
  if (mySwitch.available())
  {
    
    unsigned long value = mySwitch.getReceivedValue();
    unsigned int cval=(value & 0xff000000UL) >> 24;
    p("devicecode :  %x \n", cval );
    
    if(cval == devcode)
    {
      set(value<<8);
    }
    
    mySwitch.resetAvailable();
  }
}

void set(unsigned long value)
{
  unsigned int rval=(value & 0xff000000UL) >> 24;
  unsigned int gval=(value & 0x00ff0000UL) >> 16;
  unsigned int bval=(value & 0x0000ff00UL) >>  8;
  
  set(rval,gval,bval,value);
}

void set(unsigned int rval,unsigned int gval,unsigned int bval)
{
  unsigned long v= (rval<<24) + (gval << 16) + (bval << 8);
  set(rval,gval,bval,v);
}

void set(unsigned int rval,unsigned int gval,unsigned int bval,unsigned long value)
{
    Serial.print("This is value: ");
    Serial.println( value );
    Serial.print("Previous value: ");
    Serial.println( lastval );
    
    p("value red  :  %x \n", rval );
    p("value green:  %x \n", gval );
    p("value blue :  %x \n", bval );
    
    
    if (lastval!=value || (off && value!=0) )
    {
      if(value!=0)
      {
        lastval=value;
      }
      
      Serial.println("bingo");
      
      analogWrite(R, constrain(rval,0,0xff));
      analogWrite(G, constrain(gval,0,0xff));
      analogWrite(B, constrain(bval,0,0xff));
        
      if(value==0)
      {
        Serial.println("OFF");
        off=true;
      }
      else
      {
        Serial.println("ON");
        off=false;
      }   
    }
    else if(lastval==value)
    {
      Serial.println("reveived prevoious value ... ignoring !");
    }
}
